@@ -11,6 +11,8 @@ from contextlib import asynccontextmanager
 
 from .base import Utils, WeakContextVar, AsyncContextManager, AsyncCirculator
 from .event import DistributedEvent
+from .ntp import NTPClient
+from .transaction import Transaction
 
 
 REDIS_ERROR_RETRY_COUNT = 0x1f
@@ -131,7 +133,7 @@ class CacheClient(aioredis.Redis, AsyncContextManager):
 
         if self._pool_or_conn is None and self._pool:
 
-            if self._pool.freesize < REDIS_POOL_WATER_LEVEL_WARNING_LINE:
+            if (self._pool.maxsize - self._pool.size + self._pool.freesize) < REDIS_POOL_WATER_LEVEL_WARNING_LINE:
                 Utils.log.warning(
                     f'Redis connection pool not enough: {self._pool.freesize}({self._pool.size}/{self._pool.maxsize})'
                 )
@@ -171,7 +173,7 @@ class CacheClient(aioredis.Redis, AsyncContextManager):
 
             await self._close_conn(True)
 
-    async def execute(self, command, *args, **kwargs):
+    async def _safe_execute(self, func, *args, **kwargs):
 
         global REDIS_ERROR_RETRY_COUNT
 
@@ -183,7 +185,7 @@ class CacheClient(aioredis.Redis, AsyncContextManager):
 
                 await self._init_conn()
 
-                result = await super().execute(command, *args, **kwargs)
+                result = await func(*args, **kwargs)
 
             except (ReplyError, MaxClientsError, AuthError, ReadOnlyError) as err:
 
@@ -210,6 +212,10 @@ class CacheClient(aioredis.Redis, AsyncContextManager):
 
         return result
 
+    async def execute(self, command, *args, **kwargs):
+
+        return await self._safe_execute(super().execute, command, *args, **kwargs)
+
     def _val_encode(self, val):
 
         return Utils.pickle_dumps(val)
@@ -234,19 +240,15 @@ class CacheClient(aioredis.Redis, AsyncContextManager):
 
         return MLock(self, key, expire)
 
-    # Transaction commands
+    # TRANSACTION COMMANDS
 
     async def unwatch(self):
 
-        async with self.catch_error():
-            await self._init_conn()
-            return super().unwatch()
+        return await self._safe_execute(super().unwatch)
 
     async def watch(self, key, *keys):
 
-        async with self.catch_error():
-            await self._init_conn()
-            return super().watch()
+        return await self._safe_execute(super().watch, key, *keys)
 
     def multi_exec(self):
 
@@ -256,31 +258,23 @@ class CacheClient(aioredis.Redis, AsyncContextManager):
 
         return Pipeline(self._pool, aioredis.Redis, loop=self._pool._loop)
 
-    # Pub/Sub commands
+    # PUB/SUB COMMANDS
 
     async def subscribe(self, channel, *channels):
 
-        async with self.catch_error():
-            await self._init_conn()
-            return await super().subscribe(channel, *channels)
+        return await self._safe_execute(super().subscribe, channel, *channels)
 
     async def unsubscribe(self, channel, *channels):
 
-        async with self.catch_error():
-            await self._init_conn()
-            return await super().unsubscribe(channel, *channels)
+        return await self._safe_execute(super().unsubscribe, channel, *channels)
 
     async def psubscribe(self, pattern, *patterns):
 
-        async with self.catch_error():
-            await self._init_conn()
-            return await super().psubscribe(pattern, *patterns)
+        return await self._safe_execute(super().psubscribe, pattern, *patterns)
 
     async def punsubscribe(self, pattern, *patterns):
 
-        async with self.catch_error():
-            await self._init_conn()
-            return await super().punsubscribe(pattern, *patterns)
+        return await self._safe_execute(super().punsubscribe, pattern, *patterns)
 
     @property
     def channels(self):
@@ -354,6 +348,217 @@ class CacheClient(aioredis.Redis, AsyncContextManager):
 
         return super().delete(key, *keys)
 
+    # LIST COMMANDS
+
+    async def blpop(self, key, *keys, timeout=0, encoding=_NOTSET):
+
+        result = await super().blpop(key, *keys, timeout=timeout, encoding=encoding)
+
+        if result is not None:
+            result = [self._val_decode(val) for val in result]
+
+        return result
+
+    def _blpop(self, key, *keys, timeout=0, encoding=_NOTSET):
+
+        return super().blpop(key, *keys, timeout=timeout, encoding=encoding)
+
+    async def brpop(self, key, *keys, timeout=0, encoding=_NOTSET):
+
+        result = await super().brpop(key, *keys, timeout=timeout, encoding=encoding)
+
+        if result is not None:
+            result = [self._val_decode(val) for val in result]
+
+        return result
+
+    def _brpop(self, key, *keys, timeout=0, encoding=_NOTSET):
+
+        return super().brpop(key, *keys, timeout=timeout, encoding=encoding)
+
+    async def brpoplpush(self, sourcekey, destkey, timeout=0, encoding=_NOTSET):
+
+        result = await super().brpoplpush(sourcekey, destkey, timeout, encoding)
+
+        if result is not None:
+            result = self._val_decode(result)
+
+        return result
+
+    def _brpoplpush(self, sourcekey, destkey, timeout=0, encoding=_NOTSET):
+
+        return super().brpoplpush(sourcekey, destkey, timeout, encoding)
+
+    async def lindex(self, key, index, *, encoding=_NOTSET):
+
+        result = await super().lindex(key, index, encoding=encoding)
+
+        if result is not None:
+            result = self._val_decode(result)
+
+        return result
+
+    def _lindex(self, key, index, *, encoding=_NOTSET):
+
+        return super().lindex(key, index, encoding=encoding)
+
+    async def linsert(self, key, pivot, value, before=False):
+
+        _pivot = self._val_encode(pivot)
+        _value = self._val_encode(value)
+
+        result = await super().linsert(key, _pivot, _value, before=before)
+
+        return result
+
+    def _linsert(self, key, pivot, value, before=False):
+
+        return super().linsert(key, pivot, value, before=before)
+
+    async def lpop(self, key, *, encoding=_NOTSET):
+
+        result = await super().lpop(key, encoding=encoding)
+
+        if result is not None:
+            result = self._val_decode(result)
+
+        return result
+
+    def _lpop(self, key, *, encoding=_NOTSET):
+
+        return super().lpop(key, encoding=encoding)
+
+    async def lpush(self, key, value, *values):
+
+        _value = self._val_encode(value)
+        _values = [self._val_encode(val) for val in values]
+
+        result = await super().lpush(key, _value, *_values)
+
+        return result
+
+    def _lpush(self, key, value, *values):
+
+        return super().lpush(key, value, *values)
+
+    async def lpushx(self, key, value):
+
+        _value = self._val_encode(value)
+
+        result = await super().lpushx(key, _value)
+
+        return result
+
+    def _lpushx(self, key, value):
+
+        return super().lpushx(key, value)
+
+    async def lrange(self, key, start, stop, *, encoding=_NOTSET):
+
+        result = await super().lrange(key, start, stop, encoding=encoding)
+
+        if result is not None:
+            result = [self._val_decode(val) for val in result]
+
+        return result
+
+    def _lrange(self, key, start, stop, *, encoding=_NOTSET):
+
+        return super().lrange(key, start, stop, encoding=encoding)
+
+    async def lrem(self, key, count, value):
+
+        _value = self._val_encode(value)
+
+        result = await super().lrem(key, count, _value)
+
+        return result
+
+    def _lrem(self, key, count, value):
+
+        return super().lrem(key, count, value)
+
+    async def lset(self, key, index, value):
+
+        _value = self._val_encode(value)
+
+        result = await super().lset(key, index, _value)
+
+        return result
+
+    def _lset(self, key, index, value):
+
+        return super().lset(key, index, value)
+
+    async def ltrim(self, key, start, stop):
+
+        result = await super().ltrim(key, start, stop)
+
+        if result is not None:
+            result = [self._val_decode(val) for val in result]
+
+        return result
+
+    def _ltrim(self, key, start, stop):
+
+        return super().ltrim(key, start, stop)
+
+    async def rpop(self, key, *, encoding=_NOTSET):
+
+        result = await super().rpop(self, key, encoding=encoding)
+
+        if result is not None:
+            result = self._val_decode(result)
+
+        return result
+
+    def _rpop(self, key, *, encoding=_NOTSET):
+
+        return super().rpop(self, key, encoding=encoding)
+
+    async def rpoplpush(self, sourcekey, destkey, *, encoding=_NOTSET):
+
+        result = await super().rpoplpush(self, sourcekey, destkey, encoding=encoding)
+
+        if result is not None:
+            result = self._val_decode(result)
+
+        return result
+
+    def _rpoplpush(self, sourcekey, destkey, *, encoding=_NOTSET):
+
+        return super().rpoplpush(self, sourcekey, destkey, encoding=encoding)
+
+    async def rpush(self, key, value, *values):
+
+        _value = self._val_encode(value)
+        _values = [self._val_encode(val) for val in values]
+
+        result = await super().rpush(key, _value, *_values)
+
+        return result
+
+    def _rpush(self, key, value, *values):
+
+        _value = self._val_encode(value)
+        _values = [self._val_encode(val) for val in values]
+
+        return super().rpush(key, _value, *_values)
+
+    async def rpushx(self, key, value):
+
+        _value = self._val_encode(value)
+
+        result = await super().rpushx(key, _value)
+
+        return result
+
+    def _rpushx(self, key, value):
+
+        _value = self._val_encode(value)
+
+        return super().rpushx(key, _value)
+
 
 class MLock(AsyncContextManager):
     """基于Redis实现的分布式锁，使用with进行上下文管理
@@ -506,3 +711,93 @@ class ShareCache(AsyncContextManager):
                 await self._locker.release()
 
         self._cache = self._locker = None
+
+
+class PeriodCounter:
+
+    MIN_EXPIRE = 60
+
+    def __init__(self, ntp_client: NTPClient, cache_pool: RedisPool, time_slice: int, key_prefix: str = r''):
+
+        self._ntp_client = ntp_client
+        self._cache_pool = cache_pool
+
+        self._time_slice = time_slice
+        self._key_prefix = key_prefix
+
+    def _get_key(self, key: str = None) -> str:
+
+        time_period = Utils.math.floor(self._ntp_client.timestamp / self._time_slice)
+
+        if key is None:
+            return f'{self._key_prefix}_{time_period}'
+        else:
+            return f'{self._key_prefix}_{key}_{time_period}'
+
+    async def _incr(self, key: int, val: str) -> int:
+
+        res = None
+
+        async with self._cache_pool.get_client() as cache:
+            pipeline = cache.pipeline()
+            pipeline.incrby(key, val)
+            pipeline.expire(key, max(self._time_slice, self.MIN_EXPIRE))
+            res, _ = await pipeline.execute()
+
+        return res
+
+    async def _decr(self, key: int, val: str) -> int:
+
+        res = None
+
+        async with self._cache_pool.get_client() as cache:
+            pipeline = cache.pipeline()
+            pipeline.decrby(key, val)
+            pipeline.expire(key, max(self._time_slice, self.MIN_EXPIRE))
+            res, _ = await pipeline.execute()
+
+        return res
+
+    async def incr(self, val: int, key: str = None):
+
+        _key = self._get_key(key)
+
+        res = await self._incr(_key, val)
+
+        return res
+
+    async def incr_with_trx(self, val: int, key: str = None) -> (int, Transaction):
+
+        _key = self._get_key(key)
+
+        res = await self._incr(_key, val)
+
+        if res is not None:
+            trx = Transaction()
+            trx.add_rollback_callback(self._decr, _key, val)
+        else:
+            trx = None
+
+        return res, trx
+
+    async def decr(self, val: int, key: str = None) -> int:
+
+        _key = self._get_key(key)
+
+        res = await self._decr(_key, val)
+
+        return res
+
+    async def decr_with_trx(self, val: int, key: str = None) -> (int, Transaction):
+
+        _key = self._get_key(key)
+
+        res = await self._decr(_key, val)
+
+        if res is not None:
+            trx = Transaction()
+            trx.add_rollback_callback(self._incr, _key, val)
+        else:
+            trx = None
+
+        return res, trx
