@@ -81,7 +81,7 @@ class FormInjection:
     """表单注入器
     """
 
-    def __init__(self, form_cls=None, err_code=-1):
+    def __init__(self, form_cls=None, err_code: int = -1):
 
         if not issubclass(form_cls, Form):
             raise TypeError(r'Dot Implemented Form Interface')
@@ -92,7 +92,7 @@ class FormInjection:
     def __call__(self, func):
 
         @functools.wraps(func)
-        async def _wrapper(handler, *args, **kwargs):
+        async def _wrapper(handler: RequestBaseHandler, *args, **kwargs):
 
             form = self._form_cls(handler.request.arguments)
 
@@ -113,6 +113,43 @@ class FormInjection:
         return _wrapper
 
 
+class DebugHeader:
+    """调试信息注入器
+    """
+
+    def __call__(self, func):
+
+        @functools.wraps(func)
+        async def _wrapper(handler: RequestBaseHandler, *args, **kwargs):
+
+            if self._is_debug() is True:
+                self._receive_header(
+                    handler.get_header(r'Debug-Data')
+                )
+
+            await func(handler, *args, **kwargs)
+
+            if self._is_debug() is True:
+                handler.set_header(
+                    r'Debug-Data',
+                    self._get_debug_header()
+                )
+
+        return _wrapper
+
+    def _is_debug(self) -> bool:
+
+        raise NotImplementedError()
+
+    def _receive_header(self, data: str):
+
+        raise NotImplementedError()
+
+    def _get_debug_header(self) -> str:
+
+        raise NotImplementedError()
+
+
 class LogRequestMixin:
 
     def log_request(self):
@@ -120,7 +157,7 @@ class LogRequestMixin:
             '\n---------- request arguments ----------\n' +
             json.dumps(
                 {
-                    key: [val.decode(r'utf-8') for val in items]
+                    key: [Utils.basestring(val) for val in items]
                     for key, items in self.request.arguments.items()
                 },
                 ensure_ascii=False, indent=4
@@ -423,13 +460,13 @@ class RequestBaseHandler(RequestHandler, _BaseHandlerMixin):
 
         result = default
 
-        _argument = self.get_argument(name, None, True)
+        value = self.get_argument(name, None, True)
 
-        if result is not None:
+        if value is not None:
             try:
-                result = self.json_decode(_argument)
+                result = self.json_decode(value)
             except Exception as _:
-                self.log.debug(f'Invalid application/json argument({name}): {_argument}')
+                self.log.debug(f'Invalid application/json argument({name}): {value}')
 
         return result
 
@@ -459,10 +496,12 @@ class RequestBaseHandler(RequestHandler, _BaseHandlerMixin):
         if status_code != 200:
             self.set_status(status_code)
 
+        result = None
+
         try:
             result = self.json_encode(chunk)
         except Exception as _:
-            result = None
+            self.log.error(f'json encode error: {chunk}')
 
         return self.finish(result)
 
@@ -521,7 +560,7 @@ class DownloadAgent(RequestBaseHandler, DownloadBuffer):
         try:
             result = os.path.split(self.urlparse.urlparse(url)[2])[1]
         except Exception as _:
-            pass
+            self.log.error(f'urlparse error: {url}')
 
         return result
 
